@@ -69,11 +69,23 @@ _TRANSACTION_EVENT_DEFAULT_ATTRIBUTES = set((
         'db.statement',
         'error.class',
         'error.message',
+        'error.expected',
         'peer.hostname',
         'peer.address',
+        'graphql.field.name',
+        'graphql.field.parentType',
+        'graphql.field.path',
+        'graphql.field.returnType',
+        'graphql.operation.name',
+        'graphql.operation.type',
+        'graphql.operation.query',
+        "code.filepath",
+        "code.function",
+        "code.lineno",
+        "code.namespace",
 ))
 
-MAX_NUM_USER_ATTRIBUTES = 64
+MAX_NUM_USER_ATTRIBUTES = 128
 MAX_ATTRIBUTE_LENGTH = 255
 MAX_64_BIT_INT = 2 ** 63 - 1
 
