@@ -23,16 +23,15 @@ the global defaults or those from local agent configuration.
 
 """
 
-from newrelic.common.object_names import parse_exc_info
-import os
-import logging
 import copy
+import logging
+import os
 import re
 import threading
 
-from newrelic.core.attribute_filter import AttributeFilter
-
 import newrelic.packages.six as six
+from newrelic.common.object_names import parse_exc_info
+from newrelic.core.attribute_filter import AttributeFilter
 
 try:
     import urlparse
@@ -41,7 +40,10 @@ except ImportError:
 
 try:
     import grpc
-    from newrelic.core.infinite_tracing_pb2 import Span as _  # NOQA
+
+    from newrelic.core.infinite_tracing_pb2 import (  # pylint: disable=W0611,C0412  # noqa: F401
+        Span,
+    )
 except ImportError:
     grpc = None
 
@@ -51,11 +53,14 @@ except ImportError:
 
 DEFAULT_RESERVOIR_SIZE = 1200
 ERROR_EVENT_RESERVOIR_SIZE = 100
-SPAN_EVENT_RESERVOIR_SIZE = 1000
+SPAN_EVENT_RESERVOIR_SIZE = 2000
 
 # settings that should be completely ignored if set server side
-IGNORED_SERVER_SIDE_SETTINGS = ['utilization.logical_processors',
-        'utilization.total_ram_mib', 'utilization.billing_hostname']
+IGNORED_SERVER_SIDE_SETTINGS = [
+    "utilization.logical_processors",
+    "utilization.total_ram_mib",
+    "utilization.billing_hostname",
+]
 
 
 class _NullHandler(logging.Handler):
@@ -74,6 +79,7 @@ _logger.addHandler(_NullHandler())
 # server side configuration we create normal Settings object for new
 # sub categories we don't know about.
 
+
 class Settings(object):
     nested = False
 
@@ -88,7 +94,7 @@ class Settings(object):
 
 
 def create_settings(nested):
-    return type('Settings', (Settings,), {'nested': nested})()
+    return type("Settings", (Settings,), {"nested": nested})()
 
 
 class TopLevelSettings(Settings):
@@ -98,8 +104,7 @@ class TopLevelSettings(Settings):
     def host(self):
         if self._host:
             return self._host
-        else:
-            return default_host(self.license_key)
+        return default_host(self.license_key)
 
     @host.setter
     def host(self, value):
@@ -111,6 +116,10 @@ class AttributesSettings(Settings):
 
 
 class GCRuntimeMetricsSettings(Settings):
+    enabled = False
+
+
+class CodeLevelMetricsSettings(Settings):
     pass
 
 
@@ -277,14 +286,18 @@ class InfiniteTracingSettings(Settings):
                 "A host of %s has been extracted by interpreting the "
                 "configuration value as a URL. An update to the configuration "
                 "value is recommended.",
-                value, u.hostname)
+                value,
+                u.hostname,
+            )
             return u.hostname
         elif "/" in value or ":" in value:
             _logger.error(
                 "An invalid host (%s) was configured for infinite tracing. "
                 "Please verify that only the host name was supplied via the "
                 "infinite tracing configuration. "
-                "Falling back to infinite tracing disabled.", value)
+                "Falling back to infinite tracing disabled.",
+                value,
+            )
             return None
 
         return value
@@ -296,7 +309,8 @@ class InfiniteTracingSettings(Settings):
                 "Unable to import libraries required for infinite tracing. "
                 "Please run pip install newrelic[infinite-tracing] "
                 "to install required dependencies. "
-                "Falling back to infinite tracing disabled.")
+                "Falling back to infinite tracing disabled."
+            )
             return False
 
         return True
@@ -309,8 +323,7 @@ class EventHarvestConfigSettings(Settings):
     @property
     def report_period_ms(self):
         with self._lock:
-            return vars(_settings.event_harvest_config).get(
-                    "report_period_ms", 60 * 1000)
+            return vars(_settings.event_harvest_config).get("report_period_ms", 60 * 1000)
 
     @report_period_ms.setter
     def report_period_ms(self, value):
@@ -326,6 +339,7 @@ class EventHarvestConfigHarvestLimitSettings(Settings):
 _settings = TopLevelSettings()
 _settings.attributes = AttributesSettings()
 _settings.gc_runtime_metrics = GCRuntimeMetricsSettings()
+_settings.code_level_metrics = CodeLevelMetricsSettings()
 _settings.thread_profiler = ThreadProfilerSettings()
 _settings.transaction_tracer = TransactionTracerSettings()
 _settings.transaction_tracer.attributes = TransactionTracerAttributesSettings()
@@ -351,26 +365,22 @@ _settings.message_tracer = MessageTracerSettings()
 _settings.utilization = UtilizationSettings()
 _settings.strip_exception_messages = StripExceptionMessageSettings()
 _settings.datastore_tracer = DatastoreTracerSettings()
-_settings.datastore_tracer.instance_reporting = \
-        DatastoreTracerInstanceReportingSettings()
-_settings.datastore_tracer.database_name_reporting = \
-        DatastoreTracerDatabaseNameReportingSettings()
+_settings.datastore_tracer.instance_reporting = DatastoreTracerInstanceReportingSettings()
+_settings.datastore_tracer.database_name_reporting = DatastoreTracerDatabaseNameReportingSettings()
 _settings.heroku = HerokuSettings()
 _settings.span_events = SpanEventSettings()
 _settings.span_events.attributes = SpanEventAttributesSettings()
 _settings.transaction_segments = TransactionSegmentSettings()
-_settings.transaction_segments.attributes = \
-        TransactionSegmentAttributesSettings()
+_settings.transaction_segments.attributes = TransactionSegmentAttributesSettings()
 _settings.distributed_tracing = DistributedTracingSettings()
 _settings.serverless_mode = ServerlessModeSettings()
 _settings.infinite_tracing = InfiniteTracingSettings()
 _settings.event_harvest_config = EventHarvestConfigSettings()
-_settings.event_harvest_config.harvest_limits = \
-        EventHarvestConfigHarvestLimitSettings()
+_settings.event_harvest_config.harvest_limits = EventHarvestConfigHarvestLimitSettings()
 
 
-_settings.log_file = os.environ.get('NEW_RELIC_LOG', None)
-_settings.audit_log_file = os.environ.get('NEW_RELIC_AUDIT_LOG', None)
+_settings.log_file = os.environ.get("NEW_RELIC_LOG", None)
+_settings.audit_log_file = os.environ.get("NEW_RELIC_AUDIT_LOG", None)
 
 
 def _environ_as_int(name, default=0):
@@ -394,23 +404,23 @@ def _environ_as_bool(name, default=False):
     flag = os.environ.get(name, default)
     if default is None or default:
         try:
-            flag = not flag.lower() in ['off', 'false', '0']
+            flag = not flag.lower() in ["off", "false", "0"]
         except AttributeError:
             pass
     else:
         try:
-            flag = flag.lower() in ['on', 'true', '1']
+            flag = flag.lower() in ["on", "true", "1"]
         except AttributeError:
             pass
     return flag
 
 
-def _environ_as_set(name, default=''):
+def _environ_as_set(name, default=""):
     value = os.environ.get(name, default)
     return set(value.split())
 
 
-def _environ_as_mapping(name, default=''):
+def _environ_as_mapping(name, default=""):
     result = []
     items = os.environ.get(name, default)
 
@@ -420,19 +430,19 @@ def _environ_as_mapping(name, default=''):
     # elements, if we see one that is empty, or can't be split by ':',
     # then we know the string has an invalid format.
 
-    items = items.strip('; \t\n\r\f\v')
+    items = items.strip("; \t\n\r\f\v")
 
     if not items:
         return result
 
-    for item in items.split(';'):
+    for item in items.split(";"):
 
         try:
-            key, value = item.split(':')
+            key, value = item.split(":")
         except ValueError:
-            _logger.warning('Invalid configuration. Cannot parse: %r.'
-                    'Expected format \'key1:value1;key2:value2 ... \'.',
-                     items)
+            _logger.warning(
+                "Invalid configuration. Cannot parse: %r. Expected format 'key1:value1;key2:value2 ... '.", items
+            )
             result = []
             break
 
@@ -442,9 +452,9 @@ def _environ_as_mapping(name, default=''):
         if key and value:
             result.append((key, value))
         else:
-            _logger.warning('Invalid configuration. Cannot parse: %r.'
-                    'Expected format \'key1:value1;key2:value2 ... \'.',
-                     items)
+            _logger.warning(
+                "Invalid configuration. Cannot parse: %r. Expected format 'key1:value1;key2:value2 ... '.", items
+            )
             result = []
             break
 
@@ -455,11 +465,11 @@ def _parse_status_codes(value, target):
     items = value.split()
     for item in items:
         try:
-            negate = item.startswith('!')
+            negate = item.startswith("!")
             if negate:
                 item = item[1:]
 
-            start, end = item.split('-')
+            start, end = item.split("-")
 
             values = set(range(int(start), int(end) + 1))
 
@@ -479,78 +489,77 @@ def _parse_status_codes(value, target):
 def _parse_attributes(s):
     valid = []
     for item in s.split():
-        if '*' not in item[:-1] and len(item.encode('utf-8')) < 256:
+        if "*" not in item[:-1] and len(item.encode("utf-8")) < 256:
             valid.append(item)
         else:
-            _logger.warning('Improperly formatted attribute: %r', item)
+            _logger.warning("Improperly formatted attribute: %r", item)
     return valid
 
 
 def default_host(license_key):
     if not license_key:
-        return 'collector.newrelic.com'
+        return "collector.newrelic.com"
 
-    region_aware_match = re.match('^(.+?)x', license_key)
+    region_aware_match = re.match("^(.+?)x", license_key)
     if not region_aware_match:
-        return 'collector.newrelic.com'
+        return "collector.newrelic.com"
 
     region = region_aware_match.group(1)
-    host = 'collector.' + region + '.nr-data.net'
+    host = "collector." + region + ".nr-data.net"
     return host
 
 
 _LOG_LEVEL = {
-    'CRITICAL': logging.CRITICAL,
-    'ERROR': logging.ERROR,
-    'WARNING': logging.WARNING,
-    'INFO': logging.INFO,
-    'DEBUG': logging.DEBUG,
+    "CRITICAL": logging.CRITICAL,
+    "ERROR": logging.ERROR,
+    "WARNING": logging.WARNING,
+    "INFO": logging.INFO,
+    "DEBUG": logging.DEBUG,
 }
 
-_settings.enabled = _environ_as_bool('NEW_RELIC_ENABLED', False)
+_settings.enabled = _environ_as_bool("NEW_RELIC_ENABLED", False)
 
-_settings.feature_flag = _environ_as_set('NEW_RELIC_FEATURE_FLAG', '')
+_settings.feature_flag = _environ_as_set("NEW_RELIC_FEATURE_FLAG", "")
 
-_settings.log_level = os.environ.get('NEW_RELIC_LOG_LEVEL', 'INFO').upper()
+_settings.log_level = os.environ.get("NEW_RELIC_LOG_LEVEL", "INFO").upper()
 
 if _settings.log_level in _LOG_LEVEL:
     _settings.log_level = _LOG_LEVEL[_settings.log_level]
 else:
     _settings.log_level = logging.INFO
 
-_settings.license_key = os.environ.get('NEW_RELIC_LICENSE_KEY', None)
-_settings.api_key = os.environ.get('NEW_RELIC_API_KEY', None)
+_settings.license_key = os.environ.get("NEW_RELIC_LICENSE_KEY", None)
+_settings.api_key = os.environ.get("NEW_RELIC_API_KEY", None)
 
-_settings.ssl = _environ_as_bool('NEW_RELIC_SSL', True)
+_settings.ssl = _environ_as_bool("NEW_RELIC_SSL", True)
 
-_settings.host = os.environ.get('NEW_RELIC_HOST')
-_settings.port = int(os.environ.get('NEW_RELIC_PORT', '0'))
+_settings.host = os.environ.get("NEW_RELIC_HOST")
+_settings.port = int(os.environ.get("NEW_RELIC_PORT", "0"))
 
 _settings.agent_run_id = None
 _settings.entity_guid = None
 _settings.request_headers_map = {}
 
-_settings.proxy_scheme = os.environ.get('NEW_RELIC_PROXY_SCHEME', None)
-_settings.proxy_host = os.environ.get('NEW_RELIC_PROXY_HOST', None)
-_settings.proxy_port = int(os.environ.get('NEW_RELIC_PROXY_PORT', '0'))
-_settings.proxy_user = os.environ.get('NEW_RELIC_PROXY_USER', None)
-_settings.proxy_pass = os.environ.get('NEW_RELIC_PROXY_PASS', None)
+_settings.proxy_scheme = os.environ.get("NEW_RELIC_PROXY_SCHEME", None)
+_settings.proxy_host = os.environ.get("NEW_RELIC_PROXY_HOST", None)
+_settings.proxy_port = int(os.environ.get("NEW_RELIC_PROXY_PORT", "0"))
+_settings.proxy_user = os.environ.get("NEW_RELIC_PROXY_USER", None)
+_settings.proxy_pass = os.environ.get("NEW_RELIC_PROXY_PASS", None)
 
-_settings.ca_bundle_path = os.environ.get('NEW_RELIC_CA_BUNDLE_PATH', None)
+_settings.ca_bundle_path = os.environ.get("NEW_RELIC_CA_BUNDLE_PATH", None)
 
-_settings.app_name = os.environ.get('NEW_RELIC_APP_NAME', 'Python Application')
+_settings.app_name = os.environ.get("NEW_RELIC_APP_NAME", "Python Application")
 _settings.linked_applications = []
 
-_settings.process_host.display_name = os.environ.get(
-        'NEW_RELIC_PROCESS_HOST_DISPLAY_NAME', None)
+_settings.process_host.display_name = os.environ.get("NEW_RELIC_PROCESS_HOST_DISPLAY_NAME", None)
 
-_settings.labels = _environ_as_mapping('NEW_RELIC_LABELS', '')
+_settings.labels = _environ_as_mapping("NEW_RELIC_LABELS", "")
 
-_settings.monitor_mode = _environ_as_bool('NEW_RELIC_MONITOR_MODE', True)
+_settings.monitor_mode = _environ_as_bool("NEW_RELIC_MONITOR_MODE", True)
 
-_settings.developer_mode = _environ_as_bool('NEW_RELIC_DEVELOPER_MODE', False)
+_settings.developer_mode = _environ_as_bool("NEW_RELIC_DEVELOPER_MODE", False)
 
-_settings.high_security = _environ_as_bool('NEW_RELIC_HIGH_SECURITY', False)
+_settings.high_security = _environ_as_bool("NEW_RELIC_HIGH_SECURITY", False)
 
 _settings.attribute_filter = None
 
@@ -561,25 +570,29 @@ _settings.collect_span_events = True
 _settings.collect_analytics_events = True
 _settings.collect_custom_events = True
 
-_settings.apdex_t = _environ_as_float('NEW_RELIC_APDEX_T', 0.5)
+_settings.apdex_t = _environ_as_float("NEW_RELIC_APDEX_T", 0.5)
 _settings.web_transactions_apdex = {}
 
 _settings.capture_params = None
 _settings.ignored_params = []
 
 _settings.capture_environ = True
-_settings.include_environ = ['REQUEST_METHOD', 'HTTP_USER_AGENT',
-                              'HTTP_REFERER', 'CONTENT_TYPE',
-                              'CONTENT_LENGTH', 'HTTP_HOST', 'HTTP_ACCEPT']
+_settings.include_environ = [
+    "REQUEST_METHOD",
+    "HTTP_USER_AGENT",
+    "HTTP_REFERER",
+    "CONTENT_TYPE",
+    "CONTENT_LENGTH",
+    "HTTP_HOST",
+    "HTTP_ACCEPT",
+]
 
 _settings.max_stack_trace_lines = 50
 
 _settings.sampling_rate = 0
 
-_settings.startup_timeout = float(
-       os.environ.get('NEW_RELIC_STARTUP_TIMEOUT', '0.0'))
-_settings.shutdown_timeout = float(
-       os.environ.get('NEW_RELIC_SHUTDOWN_TIMEOUT', '2.5'))
+_settings.startup_timeout = float(os.environ.get("NEW_RELIC_STARTUP_TIMEOUT", "0.0"))
+_settings.shutdown_timeout = float(os.environ.get("NEW_RELIC_SHUTDOWN_TIMEOUT", "2.5"))
 
 _settings.beacon = None
 _settings.error_beacon = None
@@ -594,17 +607,16 @@ _settings.metric_name_rules = []
 _settings.transaction_name_rules = []
 _settings.transaction_segment_terms = []
 
-_settings.account_id = os.environ.get('NEW_RELIC_ACCOUNT_ID')
+_settings.account_id = os.environ.get("NEW_RELIC_ACCOUNT_ID")
 _settings.cross_process_id = None
-_settings.primary_application_id = \
-        os.environ.get('NEW_RELIC_PRIMARY_APPLICATION_ID', 'Unknown')
+_settings.primary_application_id = os.environ.get("NEW_RELIC_PRIMARY_APPLICATION_ID", "Unknown")
 _settings.trusted_account_ids = []
-_settings.trusted_account_key = os.environ.get('NEW_RELIC_TRUSTED_ACCOUNT_KEY')
+_settings.trusted_account_key = os.environ.get("NEW_RELIC_TRUSTED_ACCOUNT_KEY")
 _settings.encoding_key = None
 _settings.sampling_target = 10
 _settings.sampling_target_period_in_seconds = 60
 
-_settings.compressed_content_encoding = 'gzip'
+_settings.compressed_content_encoding = "gzip"
 _settings.max_payload_size_in_bytes = 1000000
 
 _settings.attributes.enabled = True
@@ -612,7 +624,7 @@ _settings.attributes.exclude = []
 _settings.attributes.include = []
 
 _settings.thread_profiler.enabled = True
-_settings.cross_application_tracer.enabled = True
+_settings.cross_application_tracer.enabled = False
 
 _settings.gc_runtime_metrics.enabled = False
 _settings.gc_runtime_metrics.top_object_count_limit = 5
@@ -624,11 +636,9 @@ _settings.transaction_events.attributes.include = []
 
 _settings.custom_insights_events.enabled = True
 
-_settings.distributed_tracing.enabled = _environ_as_bool(
-        'NEW_RELIC_DISTRIBUTED_TRACING_ENABLED', default=False)
+_settings.distributed_tracing.enabled = _environ_as_bool("NEW_RELIC_DISTRIBUTED_TRACING_ENABLED", default=True)
 _settings.distributed_tracing.exclude_newrelic_header = False
-_settings.span_events.enabled = _environ_as_bool(
-        'NEW_RELIC_SPAN_EVENTS_ENABLED', default=True)
+_settings.span_events.enabled = _environ_as_bool("NEW_RELIC_SPAN_EVENTS_ENABLED", default=True)
 _settings.span_events.attributes.enabled = True
 _settings.span_events.attributes.exclude = []
 _settings.span_events.attributes.include = []
@@ -639,7 +649,7 @@ _settings.transaction_segments.attributes.include = []
 
 _settings.transaction_tracer.enabled = True
 _settings.transaction_tracer.transaction_threshold = None
-_settings.transaction_tracer.record_sql = 'obfuscated'
+_settings.transaction_tracer.record_sql = "obfuscated"
 _settings.transaction_tracer.stack_trace_threshold = 0.5
 _settings.transaction_tracer.explain_enabled = True
 _settings.transaction_tracer.explain_threshold = 0.5
@@ -654,8 +664,7 @@ _settings.error_collector.enabled = True
 _settings.error_collector.capture_events = True
 _settings.error_collector.capture_source = False
 _settings.error_collector.ignore_classes = []
-_settings.error_collector.ignore_status_codes = _parse_status_codes(
-        '100-102 200-208 226 300-308 404', set())
+_settings.error_collector.ignore_status_codes = _parse_status_codes("100-102 200-208 226 300-308 404", set())
 _settings.error_collector.expected_classes = []
 _settings.error_collector.expected_status_codes = set()
 _settings.error_collector.attributes.enabled = True
@@ -664,18 +673,17 @@ _settings.error_collector.attributes.include = []
 
 _settings.browser_monitoring.enabled = True
 _settings.browser_monitoring.auto_instrument = True
-_settings.browser_monitoring.loader = 'rum'  # Valid values: 'full', 'none'
+_settings.browser_monitoring.loader = "rum"  # Valid values: 'full', 'none'
 _settings.browser_monitoring.loader_version = None
 _settings.browser_monitoring.debug = False
 _settings.browser_monitoring.ssl_for_http = None
-_settings.browser_monitoring.content_type = ['text/html']
+_settings.browser_monitoring.content_type = ["text/html"]
 _settings.browser_monitoring.attributes.enabled = False
 _settings.browser_monitoring.attributes.exclude = []
 _settings.browser_monitoring.attributes.include = []
 
 _settings.transaction_name.limit = None
-_settings.transaction_name.naming_scheme = os.environ.get(
-        'NEW_RELIC_TRANSACTION_NAMING_SCHEME')
+_settings.transaction_name.naming_scheme = os.environ.get("NEW_RELIC_TRANSACTION_NAMING_SCHEME")
 
 _settings.slow_sql.enabled = True
 
@@ -699,22 +707,26 @@ _settings.agent_limits.synthetics_transactions = 20
 _settings.agent_limits.data_compression_threshold = 64 * 1024
 _settings.agent_limits.data_compression_level = None
 
-_settings.infinite_tracing.trace_observer_host = os.environ.get(
-        'NEW_RELIC_INFINITE_TRACING_TRACE_OBSERVER_HOST', None)
-_settings.infinite_tracing.trace_observer_port = _environ_as_int(
-        'NEW_RELIC_INFINITE_TRACING_TRACE_OBSERVER_PORT', 443)
+_settings.infinite_tracing.trace_observer_host = os.environ.get("NEW_RELIC_INFINITE_TRACING_TRACE_OBSERVER_HOST", None)
+_settings.infinite_tracing.trace_observer_port = _environ_as_int("NEW_RELIC_INFINITE_TRACING_TRACE_OBSERVER_PORT", 443)
 _settings.infinite_tracing.ssl = True
-_settings.infinite_tracing.span_queue_size = _environ_as_int(
-        'NEW_RELIC_INFINITE_TRACING_SPAN_QUEUE_SIZE', 10000)
+_settings.infinite_tracing.span_queue_size = _environ_as_int("NEW_RELIC_INFINITE_TRACING_SPAN_QUEUE_SIZE", 10000)
 
-_settings.event_harvest_config.harvest_limits.analytic_event_data = \
-        DEFAULT_RESERVOIR_SIZE
-_settings.event_harvest_config.harvest_limits.custom_event_data = \
-        DEFAULT_RESERVOIR_SIZE
-_settings.event_harvest_config.harvest_limits.span_event_data = \
-        SPAN_EVENT_RESERVOIR_SIZE
-_settings.event_harvest_config.harvest_limits.error_event_data = \
-        ERROR_EVENT_RESERVOIR_SIZE
+_settings.event_harvest_config.harvest_limits.analytic_event_data = _environ_as_int(
+    "NEW_RELIC_ANALYTICS_EVENTS_MAX_SAMPLES_STORED", DEFAULT_RESERVOIR_SIZE
+)
+
+_settings.event_harvest_config.harvest_limits.custom_event_data = _environ_as_int(
+    "NEW_RELIC_CUSTOM_INSIGHTS_EVENTS_MAX_SAMPLES_STORED", DEFAULT_RESERVOIR_SIZE
+)
+
+_settings.event_harvest_config.harvest_limits.span_event_data = _environ_as_int(
+    "NEW_RELIC_SPAN_EVENTS_MAX_SAMPLES_STORED", SPAN_EVENT_RESERVOIR_SIZE
+)
+
+_settings.event_harvest_config.harvest_limits.error_event_data = _environ_as_int(
+    "NEW_RELIC_ERROR_COLLECTOR_MAX_EVENT_SAMPLES_STORED", ERROR_EVENT_RESERVOIR_SIZE
+)
 
 _settings.console.listener_socket = None
 _settings.console.allow_interpreter_cmd = False
@@ -736,7 +748,7 @@ _settings.debug.log_explain_plan_queries = False
 _settings.debug.log_autorum_middleware = False
 _settings.debug.record_transaction_failure = False
 _settings.debug.enable_coroutine_profiling = False
-_settings.debug.explain_plan_obfuscation = 'simple'
+_settings.debug.explain_plan_obfuscation = "simple"
 _settings.debug.disable_certificate_validation = False
 _settings.debug.log_untrusted_distributed_trace_keys = False
 _settings.debug.disable_harvest_until_shutdown = False
@@ -751,12 +763,9 @@ _settings.utilization.detect_kubernetes = True
 _settings.utilization.detect_gcp = True
 _settings.utilization.detect_pcf = True
 
-_settings.utilization.logical_processors = _environ_as_int(
-        'NEW_RELIC_UTILIZATION_LOGICAL_PROCESSORS')
-_settings.utilization.total_ram_mib = _environ_as_int(
-        'NEW_RELIC_UTILIZATION_TOTAL_RAM_MIB')
-_settings.utilization.billing_hostname = os.environ.get(
-        'NEW_RELIC_UTILIZATION_BILLING_HOSTNAME')
+_settings.utilization.logical_processors = _environ_as_int("NEW_RELIC_UTILIZATION_LOGICAL_PROCESSORS")
+_settings.utilization.total_ram_mib = _environ_as_int("NEW_RELIC_UTILIZATION_TOTAL_RAM_MIB")
+_settings.utilization.billing_hostname = os.environ.get("NEW_RELIC_UTILIZATION_BILLING_HOSTNAME")
 
 _settings.strip_exception_messages.enabled = False
 _settings.strip_exception_messages.whitelist = []
@@ -764,18 +773,17 @@ _settings.strip_exception_messages.whitelist = []
 _settings.datastore_tracer.instance_reporting.enabled = True
 _settings.datastore_tracer.database_name_reporting.enabled = True
 
-_settings.heroku.use_dyno_names = _environ_as_bool(
-        'NEW_RELIC_HEROKU_USE_DYNO_NAMES', default=True)
-_settings.heroku.dyno_name_prefixes_to_shorten = list(_environ_as_set(
-        'NEW_RELIC_HEROKU_DYNO_NAME_PREFIXES_TO_SHORTEN', 'scheduler run'))
+_settings.heroku.use_dyno_names = _environ_as_bool("NEW_RELIC_HEROKU_USE_DYNO_NAMES", default=True)
+_settings.heroku.dyno_name_prefixes_to_shorten = list(
+    _environ_as_set("NEW_RELIC_HEROKU_DYNO_NAME_PREFIXES_TO_SHORTEN", "scheduler run")
+)
 
-_settings.serverless_mode.enabled = _environ_as_bool(
-        'NEW_RELIC_SERVERLESS_MODE_ENABLED',
-        default=False)
+_settings.serverless_mode.enabled = _environ_as_bool("NEW_RELIC_SERVERLESS_MODE_ENABLED", default=False)
 _settings.aws_lambda_metadata = {}
 
 _settings.event_loop_visibility.enabled = True
 _settings.event_loop_visibility.blocking_threshold = 0.1
+_settings.code_level_metrics.enabled = True
 
 
 def global_settings():
@@ -807,11 +815,11 @@ def flatten_settings(settings):
         for key, value in vars(o).items():
             # Remove any leading underscores on keys accessed through
             # properties for reporting.
-            if key.startswith('_'):
+            if key.startswith("_"):
                 key = key[1:]
 
             if name:
-                key = '%s.%s' % (name, key)
+                key = "%s.%s" % (name, key)
 
             if isinstance(value, Settings):
                 if value.nested:
@@ -841,9 +849,9 @@ def create_obfuscated_netloc(username, password, hostname, mask):
         password = mask
 
     if username and password:
-        netloc = '%s:%s@%s' % (username, password, hostname)
+        netloc = "%s:%s@%s" % (username, password, hostname)
     elif username:
-        netloc = '%s@%s' % (username, hostname)
+        netloc = "%s@%s" % (username, hostname)
     else:
         netloc = hostname
 
@@ -866,55 +874,54 @@ def global_settings_dump(settings_object=None, serializable=False):
     # The license key is being sent already, but no point sending
     # it again.
 
-    del settings['license_key']
-    del settings['api_key']
-    del settings['encoding_key']
-    del settings['js_agent_loader']
-    del settings['js_agent_file']
+    del settings["license_key"]
+    del settings["api_key"]
+    del settings["encoding_key"]
+    del settings["js_agent_loader"]
+    del settings["js_agent_file"]
 
     # If proxy credentials are included in the settings, we obfuscate
     # them before sending, rather than deleting.
 
-    obfuscated = '****'
+    obfuscated = "****"
 
-    if settings['proxy_user'] is not None:
-        settings['proxy_user'] = obfuscated
+    if settings["proxy_user"] is not None:
+        settings["proxy_user"] = obfuscated
 
-    if settings['proxy_pass'] is not None:
-        settings['proxy_pass'] = obfuscated
+    if settings["proxy_pass"] is not None:
+        settings["proxy_pass"] = obfuscated
 
     # For the case of proxy_host we have to do a bit more work as it
     # could be a URI which includes the username and password within
     # it. What we do here is parse the value and if identified as a
     # URI, we recompose it with the obfuscated username and password.
 
-    proxy_host = settings['proxy_host']
+    proxy_host = settings["proxy_host"]
 
     if proxy_host:
         components = urlparse.urlparse(proxy_host)
 
         if components.scheme:
 
-            netloc = create_obfuscated_netloc(components.username,
-                    components.password, components.hostname, obfuscated)
+            netloc = create_obfuscated_netloc(components.username, components.password, components.hostname, obfuscated)
 
             if components.port:
-                uri = '%s://%s:%s%s' % (components.scheme, netloc,
-                        components.port, components.path)
+                uri = "%s://%s:%s%s" % (components.scheme, netloc, components.port, components.path)
             else:
-                uri = '%s://%s%s' % (components.scheme, netloc,
-                        components.path)
+                uri = "%s://%s%s" % (components.scheme, netloc, components.path)
 
-            settings['proxy_host'] = uri
+            settings["proxy_host"] = uri
 
     if serializable:
         for key, value in list(six.iteritems(settings)):
             if not isinstance(key, six.string_types):
                 del settings[key]
 
-            if (not isinstance(value, six.string_types) and
-                    not isinstance(value, float) and
-                    not isinstance(value, six.integer_types)):
+            if (
+                not isinstance(value, six.string_types)
+                and not isinstance(value, float)
+                and not isinstance(value, six.integer_types)
+            ):
                 settings[key] = repr(value)
 
     return settings
@@ -923,6 +930,7 @@ def global_settings_dump(settings_object=None, serializable=False):
 # Creation of an application settings object from global default settings
 # and any server side configuration settings.
 
+
 def apply_config_setting(settings_object, name, value, nested=False):
     """Apply a setting to the settings object where name is a dotted path.
     If there is no pre existing settings object for a sub category then
@@ -937,20 +945,19 @@ def apply_config_setting(settings_object, name, value, nested=False):
     """
 
     target = settings_object
-    fields = name.split('.', 1)
+    fields = name.split(".", 1)
 
     while len(fields) > 1:
         if not hasattr(target, fields[0]):
             setattr(target, fields[0], create_settings(nested))
         nested = False
         target = getattr(target, fields[0])
-        fields = fields[1].split('.', 1)
+        fields = fields[1].split(".", 1)
 
     default_value = getattr(target, fields[0], None)
-    if (isinstance(value, dict) and value and
-            not isinstance(default_value, dict)):
+    if isinstance(value, dict) and value and not isinstance(default_value, dict):
         for k, v in value.items():
-            k_name = '{}.{}'.format(fields[0], k)
+            k_name = "{}.{}".format(fields[0], k)
             apply_config_setting(target, k_name, v, nested=True)
     else:
         setattr(target, fields[0], value)
@@ -969,18 +976,18 @@ def fetch_config_setting(settings_object, name):
     """
 
     target = settings_object
-    fields = name.split('.', 1)
+    fields = name.split(".", 1)
 
     target = getattr(target, fields[0])
 
     while len(fields) > 1:
-        fields = fields[1].split('.', 1)
+        fields = fields[1].split(".", 1)
         target = getattr(target, fields[0])
 
     return target
 
 
-def apply_server_side_settings(server_side_config={}, settings=_settings):
+def apply_server_side_settings(server_side_config=None, settings=_settings):
     """Create a snapshot of the global default settings and overlay it
     with any server side configuration settings. Any local settings
     overrides to take precedence over server side configuration settings
@@ -993,24 +1000,26 @@ def apply_server_side_settings(server_side_config={}, settings=_settings):
     >>> settings_snapshot = apply_server_side_settings(server_config)
 
     """
+    server_side_config = server_side_config if server_side_config is not None else {}
 
     settings_snapshot = copy.deepcopy(settings)
 
     # Break out the server side agent config settings which
     # are stored under 'agent_config' key.
 
-    agent_config = server_side_config.pop('agent_config', {})
+    agent_config = server_side_config.pop("agent_config", {})
 
     # Remap as necessary any server side agent config settings.
 
-    if 'transaction_tracer.transaction_threshold' in agent_config:
-        value = agent_config['transaction_tracer.transaction_threshold']
-        if value == 'apdex_f':
-            agent_config['transaction_tracer.transaction_threshold'] = None
+    if "transaction_tracer.transaction_threshold" in agent_config:
+        value = agent_config["transaction_tracer.transaction_threshold"]
+        if value == "apdex_f":
+            agent_config["transaction_tracer.transaction_threshold"] = None
 
     # If ignore_errors exists, and either ignore_classes is not set or it is empty
-    if "error_collector.ignore_errors" in agent_config and \
-        ("error_collector.ignore_classes" not in agent_config or not agent_config["error_collector.ignore_classes"]):
+    if "error_collector.ignore_errors" in agent_config and (
+        "error_collector.ignore_classes" not in agent_config or not agent_config["error_collector.ignore_classes"]
+    ):
         # Remap to newer config key
         agent_config["error_collector.ignore_classes"] = agent_config.pop("error_collector.ignore_errors")
 
@@ -1026,21 +1035,25 @@ def apply_server_side_settings(server_side_config={}, settings=_settings):
     for (name, value) in server_side_config.items():
         apply_config_setting(settings_snapshot, name, value)
 
-    event_harvest_config = server_side_config.get('event_harvest_config', {})
-    harvest_limits = event_harvest_config.get('harvest_limits', ())
-    apply_config_setting(
-            settings_snapshot,
-            'event_harvest_config.whitelist',
-            frozenset(harvest_limits))
+    event_harvest_config = server_side_config.get("event_harvest_config", {})
+    harvest_limits = event_harvest_config.get("harvest_limits", ())
+    apply_config_setting(settings_snapshot, "event_harvest_config.whitelist", frozenset(harvest_limits))
+
+    # Override span event harvest config
+    span_event_harvest_config = server_side_config.get("span_event_harvest_config", {})
+    span_event_harvest_limit = span_event_harvest_config.get("harvest_limit", None)
+    if span_event_harvest_limit is not None:
+        apply_config_setting(
+            settings_snapshot, "event_harvest_config.harvest_limits.span_event_data", span_event_harvest_limit
+        )
 
     # This will be removed at some future point
     # Special case for account_id which will be sent instead of
     # cross_process_id in the future
 
     if settings_snapshot.cross_process_id is not None:
-        vals = [settings_snapshot.account_id,
-                settings_snapshot.application_id]
-        derived_vals = settings_snapshot.cross_process_id.split('#')
+        vals = [settings_snapshot.account_id, settings_snapshot.application_id]
+        derived_vals = settings_snapshot.cross_process_id.split("#")
 
         if len(derived_vals) == 2:
             for idx, val in enumerate(derived_vals):
@@ -1061,29 +1074,28 @@ def apply_server_side_settings(server_side_config={}, settings=_settings):
     return settings_snapshot
 
 
-def finalize_application_settings(server_side_config={}, settings=_settings):
+def finalize_application_settings(server_side_config=None, settings=_settings):
     """Overlay server-side settings and add attribute filter."""
+    server_side_config = server_side_config if server_side_config is not None else {}
 
     # Remove values from server_config that should not overwrite the
     # ones set locally
     server_side_config = _remove_ignored_configs(server_side_config)
 
-    application_settings = apply_server_side_settings(
-            server_side_config, settings)
+    application_settings = apply_server_side_settings(server_side_config, settings)
 
-    application_settings.attribute_filter = AttributeFilter(
-            flatten_settings(application_settings))
+    application_settings.attribute_filter = AttributeFilter(flatten_settings(application_settings))
 
     return application_settings
 
 
 def _remove_ignored_configs(server_settings):
-    if not server_settings.get('agent_config'):
+    if not server_settings.get("agent_config"):
         return server_settings
 
     # These settings should be ignored completely
     for ignored_setting in IGNORED_SERVER_SIDE_SETTINGS:
-        server_settings['agent_config'].pop(ignored_setting, None)
+        server_settings["agent_config"].pop(ignored_setting, None)
 
     return server_settings
 
@@ -1096,22 +1108,28 @@ def ignore_status_code(status):
 def is_expected_error(
     exc_info,
     status_code=None,
+    settings=None,
 ):
+    """Check if an error is expected based on rules matching. Default is False when settings lookup fails."""
     return error_matches_rules(
         "expected",
         exc_info,
         status_code=status_code,
+        settings=settings,
     )
 
 
 def should_ignore_error(
     exc_info,
     status_code=None,
+    settings=None,
 ):
+    """Check if an error should be ignored based on rules matching. Default is True when settings lookup fails."""
     return error_matches_rules(
         "ignore",
         exc_info,
         status_code=status_code,
+        settings=settings,
     )
 
 
@@ -1119,30 +1137,42 @@ def error_matches_rules(
     rules_prefix,
     exc_info,
     status_code=None,
+    settings=None,
 ):
+    """
+    Attempt to match exception to rules based on prefix.
+
+    rules_prefix is one of [ignore, expected]
+    exc_info is an exception tuple of (exc, val, tb)
+    status_code is an optional value or callable taking in exc_info that returns an int-like object
+    origin is either the current application or trace.
+    """
     # Delay imports to prevent lockups
-    from newrelic.api.application import application_instance
     from newrelic.core.trace_cache import trace_cache
 
-    trace = trace_cache().current_trace()
-    settings = trace and trace.settings
-
     if not settings:
-        # Retrieve application settings
-        application = application_instance()
-        settings = application and application.settings
+        # Pull from current transaction if no settings provided
+        tc = trace_cache()
+        transaction = tc.current_transaction()
+        settings = transaction and transaction.settings
 
-    # Default to global settings
-    settings = settings or global_settings()  
+        if not settings:
+            # Pull from active trace if no settings on transaction
+            trace = tc.current_trace()
+            settings = trace and trace.settings
 
-    if not settings:
-        return False
+            if not settings:
+                # Unable to find rules to match with
+                _logger.debug(
+                    "Failed to retrieve exception rules: No settings supplied, or found on transaction or trace."
+                )
+                return None
 
     # Retrieve settings based on prefix
     classes_rules = getattr(settings.error_collector, "%s_classes" % rules_prefix, set())
     status_codes_rules = getattr(settings.error_collector, "%s_status_codes" % rules_prefix, set())
 
-    module, name, fullnames, message = parse_exc_info(exc_info)
+    _, _, fullnames, _ = parse_exc_info(exc_info)
     fullname = fullnames[0]
 
     # Check class names
@@ -1155,15 +1185,14 @@ def error_matches_rules(
     # It's possible to return None, in which case no code is evaluated.
     if callable(status_code):
         status_code = status_code(*exc_info)
-    
+
     # Match status_code if it exists
     if status_code is not None:
         try:
             # Coerce into integer
             status_code = int(status_code)
         except:
-            _logger.error("Failed to coerce status code into integer. "
-                          "status_code: %s" % str(status_code))
+            _logger.error("Failed to coerce status code into integer. status_code: %s" % str(status_code))
         else:
             if status_code in status_codes_rules:
                 return True
