@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import inspect
 import logging
 import os
 import ssl
@@ -21,17 +20,12 @@ from collections import namedtuple
 
 import pytest
 
-import newrelic.packages.six as six
 from newrelic.common import certs, system_info
 from newrelic.common.agent_http import DeveloperModeClient
 from newrelic.common.encoding_utils import json_decode, serverless_payload_decode
 from newrelic.common.utilization import CommonUtilization
 from newrelic.core.agent_protocol import AgentProtocol, ServerlessModeProtocol
-from newrelic.core.config import (
-    finalize_application_settings,
-    flatten_settings,
-    global_settings,
-)
+from newrelic.core.config import finalize_application_settings, global_settings
 from newrelic.core.internal_metrics import InternalTraceContext
 from newrelic.core.stats_engine import CustomMetrics
 from newrelic.network.exceptions import (
@@ -41,6 +35,7 @@ from newrelic.network.exceptions import (
     NetworkInterfaceException,
     RetryDataForRequest,
 )
+from newrelic.packages import six
 
 Request = namedtuple("Request", ("method", "path", "params", "headers", "payload"))
 
@@ -83,16 +78,12 @@ class HttpClientRecorder(DeveloperModeClient):
         headers=None,
         payload=None,
     ):
-        request = Request(
-            method=method, path=path, params=params, headers=headers, payload=payload
-        )
+        request = Request(method=method, path=path, params=params, headers=headers, payload=payload)
         self.SENT.append(request)
         if self.STATUS_CODE:
             return self.STATUS_CODE, b""
 
-        return super(HttpClientRecorder, self).send_request(
-            method, path, params, headers, payload
-        )
+        return super(HttpClientRecorder, self).send_request(method, path, params, headers, payload)
 
     def __enter__(self):
         HttpClientRecorder.STATE += 1
@@ -169,12 +160,8 @@ def override_system_info(monkeypatch):
     IP_ADDRESS = ["127.0.0.1"]
     monkeypatch.setattr(system_info, "gethostname", lambda *args, **kwargs: HOST)
     monkeypatch.setattr(system_info, "getips", lambda *args, **kwargs: IP_ADDRESS)
-    monkeypatch.setattr(
-        system_info, "logical_processor_count", lambda *args, **kwargs: PROCESSOR_COUNT
-    )
-    monkeypatch.setattr(
-        system_info, "total_physical_memory", lambda *args, **kwargs: MEMORY
-    )
+    monkeypatch.setattr(system_info, "logical_processor_count", lambda *args, **kwargs: PROCESSOR_COUNT)
+    monkeypatch.setattr(system_info, "total_physical_memory", lambda *args, **kwargs: MEMORY)
     monkeypatch.setattr(os, "getpid", lambda *args, **kwargs: PID)
 
 
@@ -245,7 +232,11 @@ def test_send(status_code):
 def test_status_code_exceptions(status_code, expected_exc, log_level, caplog):
     caplog.set_level(logging.INFO)
     HttpClientRecorder.STATUS_CODE = status_code
-    settings = finalize_application_settings({"license_key": "123LICENSEKEY",})
+    settings = finalize_application_settings(
+        {
+            "license_key": "123LICENSEKEY",
+        }
+    )
     protocol = AgentProtocol(settings, client_cls=HttpClientRecorder)
 
     internal_metrics = CustomMetrics()
@@ -255,14 +246,16 @@ def test_status_code_exceptions(status_code, expected_exc, log_level, caplog):
 
     internal_metrics = dict(internal_metrics.metrics())
     if status_code == 413:
-        assert internal_metrics[
-            "Supportability/Python/Collector/MaxPayloadSizeLimit/analytic_event_data"
-        ] == [1, 0, 0, 0, 0, 0]
+        assert internal_metrics["Supportability/Python/Collector/MaxPayloadSizeLimit/analytic_event_data"] == [
+            1,
+            0,
+            0,
+            0,
+            0,
+            0,
+        ]
     else:
-        assert (
-            "Supportability/Python/Collector/MaxPayloadSizeLimit/analytic_event_data"
-            not in internal_metrics
-        )
+        assert "Supportability/Python/Collector/MaxPayloadSizeLimit/analytic_event_data" not in internal_metrics
 
     assert len(HttpClientRecorder.SENT) == 1
     request = HttpClientRecorder.SENT[0]
@@ -275,17 +268,13 @@ def test_status_code_exceptions(status_code, expected_exc, log_level, caplog):
 
 
 def test_protocol_http_error_causes_retry():
-    protocol = AgentProtocol(
-        finalize_application_settings(), client_cls=HttpClientException
-    )
+    protocol = AgentProtocol(finalize_application_settings(), client_cls=HttpClientException)
     with pytest.raises(RetryDataForRequest):
         protocol.send("analytic_event_data")
 
 
 def test_protocol_context_manager():
-    protocol = AgentProtocol(
-        finalize_application_settings(), client_cls=HttpClientRecorder
-    )
+    protocol = AgentProtocol(finalize_application_settings(), client_cls=HttpClientRecorder)
     with protocol:
         assert HttpClientRecorder.STATE == 1
 
@@ -293,9 +282,7 @@ def test_protocol_context_manager():
 
 
 def test_close_connection():
-    protocol = AgentProtocol(
-        finalize_application_settings(), client_cls=HttpClientRecorder
-    )
+    protocol = AgentProtocol(finalize_application_settings(), client_cls=HttpClientRecorder)
     protocol.close_connection()
     assert HttpClientRecorder.STATE == -1
 
@@ -310,7 +297,7 @@ def connect_payload_asserts(
     with_kubernetes=True,
 ):
     payload_data = payload[0]
-    assert type(payload_data["agent_version"]) is type(u"")
+    assert isinstance(payload_data["agent_version"], type(u""))
     assert payload_data["app_name"] == PAYLOAD_APP_NAME
     assert payload_data["display_host"] == DISPLAY_NAME
     assert payload_data["environment"] == ENVIRONMENT
@@ -323,16 +310,10 @@ def connect_payload_asserts(
     assert payload_data["pid"] == PID
     assert len(payload_data["security_settings"]) == 2
     assert payload_data["security_settings"]["capture_params"] == CAPTURE_PARAMS
-    assert payload_data["security_settings"]["transaction_tracer"] == {
-        "record_sql": RECORD_SQL
-    }
+    assert payload_data["security_settings"]["transaction_tracer"] == {"record_sql": RECORD_SQL}
     assert len(payload_data["settings"]) == 2
-    assert payload_data["settings"]["browser_monitoring.loader"] == (
-        BROWSER_MONITORING_LOADER
-    )
-    assert payload_data["settings"]["browser_monitoring.debug"] == (
-        BROWSER_MONITORING_DEBUG
-    )
+    assert payload_data["settings"]["browser_monitoring.loader"] == (BROWSER_MONITORING_LOADER)
+    assert payload_data["settings"]["browser_monitoring.debug"] == (BROWSER_MONITORING_DEBUG)
 
     utilization_len = 5
 
@@ -344,9 +325,7 @@ def connect_payload_asserts(
     else:
         assert "ip_address" not in payload_data["utilization"]
 
-    utilization_len = utilization_len + any(
-        [with_aws, with_pcf, with_gcp, with_azure, with_docker, with_kubernetes]
-    )
+    utilization_len = utilization_len + any([with_aws, with_pcf, with_gcp, with_azure, with_docker, with_kubernetes])
     assert len(payload_data["utilization"]) == utilization_len
     assert payload_data["utilization"]["hostname"] == HOST
 
@@ -413,9 +392,7 @@ def connect_payload_asserts(
         (True, True, True, True, False, True, True),
     ],
 )
-def test_connect(
-    with_aws, with_pcf, with_gcp, with_azure, with_docker, with_kubernetes, with_ip
-):
+def test_connect(with_aws, with_pcf, with_gcp, with_azure, with_docker, with_kubernetes, with_ip):
     global AWS, AZURE, GCP, PCF, BOOT_ID, DOCKER, KUBERNETES, IP_ADDRESS
     if not with_aws:
         AWS = Exception
@@ -457,7 +434,11 @@ def test_connect(
         }
     )
     protocol = AgentProtocol.connect(
-        APP_NAME, LINKED_APPS, ENVIRONMENT, settings, client_cls=HttpClientRecorder,
+        APP_NAME,
+        LINKED_APPS,
+        ENVIRONMENT,
+        settings,
+        client_cls=HttpClientRecorder,
     )
 
     # verify there are exactly 3 calls to HttpClientRecorder
@@ -507,7 +488,7 @@ def test_connect(
 def test_connect_metadata(monkeypatch):
     monkeypatch.setenv("NEW_RELIC_METADATA_FOOBAR", "foobar")
     monkeypatch.setenv("_NEW_RELIC_METADATA_WRONG", "wrong")
-    protocol = AgentProtocol.connect(
+    AgentProtocol.connect(
         APP_NAME,
         LINKED_APPS,
         ENVIRONMENT,
@@ -539,9 +520,7 @@ def test_serverless_protocol_connect():
 
 def test_serverless_protocol_finalize(capsys):
     protocol = ServerlessModeProtocol(
-        finalize_application_settings(
-            {"aws_lambda_metadata": {"foo": "bar", "agent_version": "x"}}
-        )
+        finalize_application_settings({"aws_lambda_metadata": {"foo": "bar", "agent_version": "x"}})
     )
     response = protocol.send("metric_data", (1, 2, 3))
     assert response is None
@@ -575,7 +554,13 @@ def test_audit_logging():
     assert len(audit_log_contents) > 2
 
 
-@pytest.mark.parametrize("ca_bundle_path", (None, "custom",))
+@pytest.mark.parametrize(
+    "ca_bundle_path",
+    (
+        None,
+        "custom",
+    ),
+)
 def test_ca_bundle_path(monkeypatch, ca_bundle_path):
     # Pretend CA certificates are not available
     class DefaultVerifyPaths(object):
@@ -593,9 +578,7 @@ def test_ca_bundle_path(monkeypatch, ca_bundle_path):
 
 
 def test_max_payload_size_limit():
-    settings = finalize_application_settings(
-        {"max_payload_size_in_bytes": 0, "port": -1}
-    )
+    settings = finalize_application_settings({"max_payload_size_in_bytes": 0, "port": -1})
     protocol = AgentProtocol(settings, host="localhost")
     with pytest.raises(DiscardDataForRequest):
         protocol.send("metric_data")
