@@ -16,6 +16,7 @@ import pytest
 
 from testing_support.fixtures import (validate_transaction_metrics,
     validate_transaction_errors, override_application_settings)
+from testing_support.validators.validate_code_level_metrics import validate_code_level_metrics
 
 def target_application():
     # We need to delay Flask application creation because of ordering
@@ -51,6 +52,12 @@ _test_application_app_middleware_scoped_metrics = [
 @validate_transaction_errors(errors=[])
 @validate_transaction_metrics('_test_middleware:index_page',
         scoped_metrics=_test_application_app_middleware_scoped_metrics)
+@validate_code_level_metrics("_test_middleware", "index_page")
+@validate_code_level_metrics("_test_middleware", "before_first_request")
+@validate_code_level_metrics("_test_middleware", "before_request")
+@validate_code_level_metrics("_test_middleware", "after_request")
+@validate_code_level_metrics("_test_middleware", "teardown_request")
+@validate_code_level_metrics("_test_middleware", "teardown_appcontext")
 def test_application_app_middleware():
     application = target_application()
     response = application.get('/middleware')
